#
# raccp.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2.
#

require 'racc/parser'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @yydebug   = racc.d_parse && Racc_debug_parser
      @interf    = racc.interf
      @ruletable = racc.ruletable
    end


    attr :classname

    def parse( str )
      @scanner = RaccScanner.new( str )
      @scanner.debug = @yydebug

      do_parse
    end


    private


    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      if val.respond_to? :id2name then
        v = val.id2name
      elsif String === val then
        v = val
      else
        v = val.inspect
      end
      raise ParseError, "#{@scanner.lineno}: unexpected token '#{v}'"
    end

##### racc 1.1.4 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 6, 21, :_reduce_1,
 0, 22, :_reduce_none,
 2, 22, :_reduce_none,
 3, 24, :_reduce_4,
 1, 24, :_reduce_none,
 2, 24, :_reduce_6,
 2, 24, :_reduce_7,
 2, 25, :_reduce_8,
 3, 25, :_reduce_9,
 3, 26, :_reduce_10,
 3, 26, :_reduce_11,
 1, 29, :_reduce_none,
 2, 29, :_reduce_none,
 2, 30, :_reduce_14,
 2, 30, :_reduce_15,
 2, 30, :_reduce_16,
 1, 28, :_reduce_17,
 2, 28, :_reduce_18,
 1, 27, :_reduce_19,
 1, 27, :_reduce_20,
 1, 23, :_reduce_21,
 0, 23, :_reduce_none,
 1, 31, :_reduce_23,
 2, 31, :_reduce_24,
 2, 31, :_reduce_25,
 2, 31, :_reduce_26,
 1, 32, :_reduce_none,
 1, 32, :_reduce_28,
 2, 32, :_reduce_29,
 1, 32, :_reduce_30 ]

racc_reduce_n = 31

racc_shift_n = 51

racc_action_table = [
    16,    12,     1,    13,    14,     7,    15,     9,    11,    48,
    45,    16,    44,    38,    40,    41,    42,    15,    16,    16,
    16,    16,    16,    47,    15,    15,    15,    15,    15,    16,
    16,    16,    16,    16,    16,    15,    15,    15,    15,    15,
    15,    16,    37,     6,    19,    20,    22,    15,    34,    19,
    20,    22,    19,    20,    22,    19,    20,    22,     4,     3,
    50 ]

racc_action_check = [
    26,     5,     0,     5,     5,     5,    26,     5,     5,    29,
    27,    13,    26,    26,    26,    26,    26,    13,    33,    20,
     7,    28,    41,    28,    33,    20,     7,    28,    41,    12,
    36,    14,    22,    19,    32,    12,    36,    14,    22,    19,
    32,    18,    24,     4,    24,    24,    24,    18,    21,    21,
    21,    21,    11,    11,    11,     9,     9,     9,     2,     1,
    46 ]

racc_action_default = [
   -31,   -31,   -31,    -2,   -31,   -31,    51,   -31,    -3,   -31,
    -5,   -31,   -22,   -31,   -31,   -20,   -19,   -17,    -7,   -31,
   -31,   -31,   -31,   -12,   -31,   -23,   -21,   -31,   -31,   -31,
    -6,   -18,   -14,   -15,   -10,   -13,   -16,   -11,   -26,   -24,
   -28,   -31,   -30,   -27,   -25,    -1,   -31,    -4,    -8,   -29,
    -9 ]

racc_action_pointer = [
     0,    56,    58,   nil,    43,    -3,   nil,    17,   nil,    43,
   nil,    40,    26,     8,    28,   nil,   nil,   nil,    38,    30,
    16,    37,    29,   nil,    32,   nil,    -3,     5,    18,     0,
   nil,   nil,    31,    15,   nil,   nil,    27,   nil,   nil,   nil,
   nil,    19,   nil,   nil,   nil,   nil,    51,   nil,   nil,   nil,
   nil ]

racc_goto_table = [
    25,    29,    30,    21,    35,    24,    31,    35,     2,    10,
    18,    28,     8,    27,    43,     5,    46,    26,    39,   nil,
    31,    31,    32,    33,    31,    36,   nil,   nil,   nil,    49 ]

racc_goto_check = [
     7,     7,     7,     9,    10,     9,     7,    10,     1,     6,
     8,     5,     4,     3,     7,     2,     7,    11,    12,   nil,
     7,     7,     8,     8,     7,     8,   nil,   nil,   nil,     7 ]

racc_goto_pointer = [
   nil,     8,    12,     1,     7,    -2,     4,   -12,     3,    -6,
   -17,     5,    -8 ]

racc_goto_default = [
   nil,   nil,   nil,   nil,   nil,   nil,   nil,    17,   nil,   nil,
    23,   nil,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :XCLASS => 2,
 :XSYMBOL => 3,
 :XRULE => 4,
 :XEND => 5,
 :XCONV => 6,
 :XSTART => 7,
 :XTOKEN => 8,
 :STRING => 9,
 :XPRECHIGH => 10,
 :XPRECLOW => 11,
 :XLEFT => 12,
 :XRIGHT => 13,
 :XNONASSOC => 14,
 ";" => 15,
 ":" => 16,
 "|" => 17,
 "=" => 18,
 :ACTION => 19 }

racc_nt_base = 20

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n ]

Racc_debug_parser = false

##### racc system variables end #####


 # reduce 0 omitted

 module_eval( <<'.,.,', '(build)', 83 )
  def _reduce_1( val, _values, result )
      @interf.end_register_rule
      @classname = val[1].id2name
   result
  end
.,.,

 # reduce 2 omitted

 # reduce 3 omitted

 module_eval( <<'.,.,', '(build)', 94 )
  def _reduce_4( val, _values, result )
      @interf.end_register_conv
   result
  end
.,.,

 # reduce 5 omitted

 module_eval( <<'.,.,', '(build)', 100 )
  def _reduce_6( val, _values, result )
      @interf.register_start val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 104 )
  def _reduce_7( val, _values, result )
      @interf.register_token val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 109 )
  def _reduce_8( val, _values, result )
      @interf.register_conv( val[0], val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 113 )
  def _reduce_9( val, _values, result )
      @interf.register_conv( val[1], val[2] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 118 )
  def _reduce_10( val, _values, result )
      @interf.end_register_prec( true )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 122 )
  def _reduce_11( val, _values, result )
      @interf.end_register_prec( false )
   result
  end
.,.,

 # reduce 12 omitted

 # reduce 13 omitted

 module_eval( <<'.,.,', '(build)', 132 )
  def _reduce_14( val, _values, result )
      @interf.register_prec( :Left, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 136 )
  def _reduce_15( val, _values, result )
      @interf.register_prec( :Right, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 140 )
  def _reduce_16( val, _values, result )
      @interf.register_prec( :Nonassoc, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 145 )
  def _reduce_17( val, _values, result )
      result = val
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 149 )
  def _reduce_18( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 154 )
  def _reduce_19( val, _values, result )
      result = @interf.get_symbol( result )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 158 )
  def _reduce_20( val, _values, result )
      result = @interf.get_symbol( eval '"' + val[0] + '"' )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 163 )
  def _reduce_21( val, _values, result )
      unless result.empty? then
        @interf.register_from_array result
      end
   result
  end
.,.,

 # reduce 22 omitted

 module_eval( <<'.,.,', '(build)', 172 )
  def _reduce_23( val, _values, result )
      result = val
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 176 )
  def _reduce_24( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 180 )
  def _reduce_25( val, _values, result )
      unless result.empty? then
        @interf.register_from_array result
      end
      result.clear
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 187 )
  def _reduce_26( val, _values, result )
      pre = result.pop
      unless result.empty? then
        @interf.register_from_array result
      end
      result = [pre]
   result
  end
.,.,

 # reduce 27 omitted

 module_eval( <<'.,.,', '(build)', 198 )
  def _reduce_28( val, _values, result )
      result = OrMark.new( @scanner.lineno )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 202 )
  def _reduce_29( val, _values, result )
      result = Prec.new( val[1], @scanner.lineno )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 206 )
  def _reduce_30( val, _values, result )
      result = Action.new( *result )
   result
  end
.,.,

  end   # class RaccParser

end   # module Racc
